@@ -155,8 +155,9 @@ class DataStream(object):
     def get(self, listener):
         return self._queues.get(listener).get()
 
+    @abc.abstractmethod
     def size(self):
-        return self._size
+        pass
 
     def put(self, item):
         """this is where computation results are passed to."""
@@ -180,14 +181,22 @@ class DataStream(object):
 
 class Source(DataStream):
 
-    def __init__(self, source):
+    def __init__(self, items=None):
         super(Source, self).__init__()
-        self._size = len(source)
-        self.source = source
+        self.items = items
         self.input_streams = []
 
+    def set_input_items(self, items):
+        self.items = items
+
+    def set_input_item(self, item):
+        self.items = [item]
+
+    def size(self):
+        return len(self.items)
+
     def _start_computation(self):
-        for i, item in enumerate(self.source):
+        for i, item in enumerate(self.items):
             self.put(EnumeratedItem(i, item))
 
 
@@ -199,7 +208,6 @@ class Computation(DataStream):
         self.debug = debug
         self.f = f
         self.input_streams = self._setup_inputs(inputs)
-        self._size = self.compute_size()
 
     def _setup_inputs(self, inputs):
         streams = []
@@ -214,10 +222,6 @@ class Computation(DataStream):
             streams.append(stream)
         return streams
 
-    @abc.abstractmethod
-    def compute_size(self):
-        pass
-
     @abc.abstractmethod
     def _start_computation(self):
         pass
@@ -233,6 +237,13 @@ class Computation(DataStream):
         Engine.run_async(self.f, args, forward_result, self.run_local, self.debug)
 
 
+"""
+todo: cleanup size computation so it is only triggered when computation starts
+      more tests using a "delayed" Source
+      separate wf setup and running in envipy
+"""
+
+
 class OutputNode(DataStream):
 
     def __init__(self, input_):
@@ -248,7 +259,7 @@ class OutputNode(DataStream):
         self.input_ = input_
 
     def size(self):
-        return self.input_._size
+        return self.input_.size()
 
     def start_computations(self, do_not_reraise_exceptions=True):
         """
@@ -301,7 +312,7 @@ class _UpdateManagerForJoin(object):
 
 class JoinComputation(Computation):
 
-    def compute_size(self):
+    def size(self):
         return reduce(lambda x, y: x * y, (a.size() for a in self.input_streams))
 
     def _start_computation(self):
@@ -331,7 +342,7 @@ class SingleInputComputation(Computation):
 
 class SummarizeComputation(SingleInputComputation):
 
-    def compute_size(self):
+    def size(self):
         return 1
 
     def _start_computation(self):
@@ -351,7 +362,7 @@ class SummarizeComputation(SingleInputComputation):
 
 class FullStreamComputation(SingleInputComputation):
 
-    def compute_size(self):
+    def size(self):
         return self.input_streams[0].size()
 
     def _start_computation(self):
@@ -385,7 +396,7 @@ class ZipComputation(Computation):
     ConstantSource class.
     """
 
-    def compute_size(self):
+    def size(self):
         sizes = set(s.size() for s in self.input_streams)
         if 1 in sizes:
             sizes.remove(1)
