@@ -298,7 +298,12 @@ class _CachedFunction(object):
         return obj
 
     def __call__(self, *args, **kw):
-        hash_code = self._compute_hash(args + tuple(sorted(kw.items())))
+        all_args = args + tuple(sorted(kw.items()))
+        try:
+            hash_code = self._compute_hash(all_args)
+        except RuntimeError:
+            raise Exception("could not compute hash for %r. maybe you should register your own "
+                            "handler" % (all_args,))
         if self._contains(hash_code):
             with self._lock:
                 self._hit_counter.value += 1
